    program Draht;
{+}
{ Demonstrationsprogramm fuer zentralperspektivische Abbildung und
{ fuer Bewegungssimulation.
{-}
    const
      c1 = 80.0; c2 = 140.0; c3 = 40.0;         { Objekt }
      Kmax = 6;
      mx = 158; my = 128;                        { Bildschirmmitte }
      verzerr = 0.66667;                    { Bildschirmverzerrung }
    type
      realarray3 = array[1..3] of real;
      realarray2 = array[1..2] of real;
      string80 = string[80];
    var
      x, y : array[1..kmax] of real;             { Objekt }
      u1, u2, v1, v2, it, k, k1, k2 : integer;
      rep, linemask : integer;
      bildu :realarray2; obs, obj : realarray3;
      init : boolean;
      character : char;

    procedure Zentpers(x : realarray3; { Objektkoordinaten }
                       c : realarray3; { Projektionszentrum }
                       init : boolean; { = true fuer neues c }
                       var u : realarray2);      { Bildkoordinaten}
{+}
{ Bildet die Objektpunkt x[] zentralperspektivisch in den Bildpunkt
{ u[] ab; c[] ist das Projektionszentrum.
{-}
    const
      r : array[1..8] of real = (0.0, 0.0, 0.0, 0.0,
                                 0.0, 0.0, 0.0, 0.0);
      cc : real = 0.0;
      usub : real = 0.0; vsub : real = 0.0; wsub : real = 0.0;
    var
      pp, w : real;
    begin
      if init then
      begin
        pp := Sqr(c[1]) + Sqr(c[2]); cc := Sqrt(pp + Sqr(c[3]));
        pp := Sqrt(pp);
        if pp > 1.0E-20 then
        begin
          r[5] := c[1] / pp; r[2] := - c[2] / pp; r[7] := - pp / cc;
          r[8] := c[3] / cc;
        end
        else
        begin
          r[2] := 0.0; r[5] := 0.0; r[7] := 0.0; r[8] := 1.0
        end;
        r[1] :=   r[5] * r[8]; r[3] := - r[5] * r[7];
        r[4] := - r[2] * r[8]; r[6] :=   r[2] * r[7];
        usub := r[1] * c[1] + r[4] * c[2] + r[7] * c[3];
        vsub := r[2] * c[1] + r[5] * c[2];
        wsub := r[3] * c[1] + r[6] * c[2] + r[8] * c[3];
      end;
      w := r[3] * x[1] + r[6] * x[2] + r[8] * x[3] - wsub;
      if Abs(w) > 1.0E-20 then
      begin
        w:= - cc / w;
        u[1] := (r[1] * x[1] + r[4] * x[2] + r[7] * x[3] - usub) * w;
        u[2] := (r[2] * x[1] + r[5] * x[2] - vsub) * w;
      end
      else
      begin
        u[1] := 0.0; u[2] := 0.0;
      end;
    end;           { Ende der Procedur Zentpers }

    procedure Beobachter(var c : realarray3; t : integer);
{+}
{ Liefert fuer den Zeitpunkt t einen Beobachterstandpunkt.
{-}
    const
      frequenz = 0.03142; strahl = 600.0;
    var
      azi, ele, co, tmp : real;
    begin
      azi := frequenz * t; tmp := Cos(azi); ele := tmp * 0.6;
      co  := Cos( ele );
      c[1] := strahl * tmp * co;
      c[2] := strahl * Sin( azi ) * co;
      c[3] := strahl * Sin( ele );
    end;

    procedure Clearscreen;
{+}
{ Loescht den gesamten Bildschirm.
{-}
    begin
      write(chr(27),'M',chr(12));
    end;

    procedure Draw(xx1,y1,xx2,y2 : integer;
                   var l : integer);
{+}
{ Zeichnen einer Linie.
{-}
    begin
      write(chr(27),'D');
      write(chr(xx1 mod 256),chr(xx1 div 256),chr(y1));
      write(chr(xx2 mod 256),chr(xx2 div 256),chr(y2));
    end;

    procedure Nohires;
{+}
{ Schaltet den Bildschirm auf Textmodus.
{-}
    begin
      Clearscreen;
      write(chr(27),chr(96),chr(0));
    end;

{+}
{ Beginn des Hauptteils.
{-}
    begin
      linemask := $FFFF;
      x[1] :=   c1; y[1] :=  0.0; x[2] :=  0.0; y[2] :=   c2;
      x[3] := - c1; y[3] :=   c2; x[4] := - c1; y[4] := - c2;
      x[5] :=   c1; y[5] := - c2; x[6] :=   c1; y[6] :=  0.0;
      write(chr(27),chr(96),chr(0));

      for rep := 1 to 200 do
      begin
        Clearscreen;
        Beobachter(obs, rep); init := true;
        begin
          for k1 := 1 to Pred(kmax) do
          begin
            k2 := Succ(k1);
            obj[1] := x[k1]; obj[2] := y[k1]; obj[3] := - c3;
            Zentpers(obj, obs, init, bildu); init := false;
            u1 := Round(bildu[1]) + mx;
            v1 := Round(bildu[2] * verzerr) + my;
            obj[1] := x[k2]; obj[2] := y[k2];
            Zentpers(obj, obs, false, bildu);
            u2 := Round(bildu[1]) + mx;
            v2 := Round(bildu[2] * verzerr) + my;
            Draw(u1, v1, u2, v2, linemask);
            u1 := u2; v1 := v2;
            obj[3] := c3;
            Zentpers(obj, obs, false, bildu);
            u2 := Round(bildu[1]) + mx;
            v2 := Round(bildu[2] * verzerr) + my;
            Draw(u1, v1, u2, v2, linemask);
            u1 := u2; v1 := v2;
            obj[1] := x[k1]; obj[2] := y[k1];
            Zentpers(obj, obs, false, bildu);
            u2 := Round(bildu[1]) + mx;
            v2 := Round(bildu[2] * verzerr) + my;
            Draw(u1, v1, u2, v2, linemask);
          end;
        end;
{+}
{ Umspewichern des Arbeitsbildbereiches in den Bildbereich.
{-}
        if rep mod 2 = 0
           then begin write(chr(27),chr(96),chr(1)); end
           else begin write(chr(27),chr(96),chr(4)); end;
      end;
      Read(kbd, character);
      Nohires;
    end.
